/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.ddv.converter;

import dk.sundhedsdatastyrelsen.cbs.datafetcher.exceptions.DDVException;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.EffectuatedVaccination;
import dk.vaccinationsregister.schemas._2013._12._01.ATCType;
import dk.vaccinationsregister.schemas._2013._12._01.EffectuationOverviewType;
import dk.vaccinationsregister.schemas._2013._12._01.SSIDrugType;
import org.apache.log4j.Logger;
import org.springframework.core.convert.converter.Converter;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;

import static dk.sundhedsdatastyrelsen.cbs.dto.impl.vaccination.EffectuatedVaccinationBuilder.effectuatedVaccination;
import static java.util.Optional.ofNullable;

public class EffectuatedVaccinationConverter implements Converter<EffectuationOverviewType, EffectuatedVaccination> {
    private static final Logger log = Logger.getLogger(EffectuatedVaccinationConverter.class);

    @Override
    @Nullable
    public EffectuatedVaccination convert(@NonNull final EffectuationOverviewType source) {
        if (source.getEffectuatedDateTime() == null) {
            log.debug("Discarding EffectuationOverview element due to it having no EffectuatedDateTime. (this is a planned effectuation)");
            return null; //irrelevant data
        }
        if (source.getVaccine() == null) throw new DDVException(VaccinationConverter.INVALID_DDV_DATA_PREFIX + "EffectuationOverview from DDV has no Vaccine element");
        ATCType atc = source.getVaccine().getATC();
        String atcCode = atc.getCode();
        if (atcCode == null) throw new DDVException(VaccinationConverter.INVALID_DDV_DATA_PREFIX + "EffectuationOverview.Vaccine.ATC from DDV has no Code element");
        return effectuatedVaccination(
            source.getVaccine().getVaccineIdentifier(),
            ofNullable(source.getSSIDrug()).map(SSIDrugType::getDrugIdentifier).orElse(null),
            source.getDrugName(),
            atcCode,
            source.getEffectuatedDateTime().toLocalDateTime()
        ).build();
    }
}
