/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.test;

import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResults;
import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.Vaccination;
import org.springframework.lang.NonNull;

import java.time.LocalDateTime;
import java.util.concurrent.atomic.AtomicLong;

import static dk.sundhedsdatastyrelsen.cbs.dto.impl.lab.LabResultBuilder.antigenResult;
import static dk.sundhedsdatastyrelsen.cbs.dto.impl.lab.LabResultBuilder.pcrResult;
import static dk.sundhedsdatastyrelsen.cbs.dto.impl.lab.LabResultsBuilder.cpr;
import static dk.sundhedsdatastyrelsen.cbs.dto.impl.vaccination.EffectuatedVaccinationBuilder.effectuatedVaccination;
import static dk.sundhedsdatastyrelsen.cbs.dto.impl.vaccination.VaccinationBuilder.vaccination;

public final class Data {
    private Data() {}

    private static final RandomData RANDOM_DATA = new RandomData("CBS".hashCode());
    private static final String DEFAULT_CPR = RANDOM_DATA.cpr();
    public static final class Vaccinations {
        private Vaccinations() {}
        public static final Vaccination EMPTY = vaccination(DEFAULT_CPR, false).build();
        private static final AtomicLong NEXT_IDENTIFIER = new AtomicLong();
        private static final String DEFAULT_DRUG_NAME = "Drug name";
        private static final String DEFAULT_ATC_CODE = "ATC code";

        @NonNull
        public static Vaccination completed(@NonNull final LocalDateTime completionDate) {
            return vaccination(DEFAULT_CPR, true)
                .add(effectuatedVaccination(
                    NEXT_IDENTIFIER.incrementAndGet(),
                    RANDOM_DATA.nextBoolean() ? RANDOM_DATA.nextLong() : null,
                    DEFAULT_DRUG_NAME,
                    DEFAULT_ATC_CODE,
                    completionDate).build()).build();
        }
    }

    public static final class Results {
        private Results() {}
        public static final LabResults EMPTY = cpr(DEFAULT_CPR).build();
        private static final AtomicLong NEXT_IDENTIFIER = new AtomicLong();

        @NonNull
        public static LabResults positivePcr(@NonNull final LocalDateTime resultTime) {
            return cpr(DEFAULT_CPR)
                .add(pcrResult(NEXT_IDENTIFIER.incrementAndGet(), resultTime.minusHours(RANDOM_DATA.nextInt(4, 42)), resultTime).positive().build())
                .build();
        }
        @NonNull
        public static LabResults positiveAntigen(@NonNull final LocalDateTime resultTime) {
            return cpr(DEFAULT_CPR)
                .add(antigenResult(NEXT_IDENTIFIER.incrementAndGet(), resultTime.minusHours(RANDOM_DATA.nextInt(4, 42)), resultTime).positive().build())
                .build();
        }
        @NonNull
        public static LabResults positiveAntigenNegativePcr(@NonNull final LocalDateTime resultTime) {
            return cpr(DEFAULT_CPR)
                .add(antigenResult(NEXT_IDENTIFIER.incrementAndGet(), resultTime.minusHours(RANDOM_DATA.nextInt(4, 42)), resultTime).positive().build())
                .add(pcrResult(NEXT_IDENTIFIER.incrementAndGet(), resultTime.minusHours(RANDOM_DATA.nextInt(4, 42)), resultTime.plusNanos(1)).negative().build())
                .build();
        }

    }
}
