/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.test;

import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResult;
import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResults;
import dk.sundhedsdatastyrelsen.cbs.test.Data.Results;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.springframework.lang.NonNull;

import java.time.LocalDateTime;
import java.util.Optional;
import java.util.stream.Stream;

import static dk.sundhedsdatastyrelsen.cbs.test.Data.Results.EMPTY;
import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;
import static org.junit.jupiter.api.Assertions.assertEquals;

class ResultsTest {

    private static final RandomData RANDOM_DATA = new RandomData();
    private static final int MAX_TEST = 100;

    @Test
    @SuppressWarnings("ExcessiveLambdaUsage")
    void empty() {
        assertDoesNotThrow(() -> EMPTY);
    }

    @ParameterizedTest
    @MethodSource("source")
    void positiveAntigen(@NonNull final LocalDateTime dateTime) {
        assertEquals(
            dateTime,
            Optional.of(Results.positiveAntigen(dateTime))
                .map(LabResults::mostRecentPositiveAntigenResult)
                .map(LabResult::resultDateTime)
                .orElse(null));
    }

    @ParameterizedTest
    @MethodSource("source")
    void positivePcr(@NonNull final LocalDateTime dateTime) {
        assertEquals(
            dateTime,
            Optional.of(Results.positivePcr(dateTime))
                .map(LabResults::mostRecentPositivePcrResult)
                .map(LabResult::resultDateTime)
                .orElse(null));
    }

    @ParameterizedTest
    @MethodSource("source")
    void positiveAntigenNegativePcr(@NonNull final LocalDateTime dateTime) {
        final LabResults results = Results.positiveAntigenNegativePcr(dateTime);
        assertEquals(
            dateTime,
            Optional.of(results)
                .map(LabResults::mostRecentPositiveAntigenResult)
                .map(LabResult::resultDateTime)
                .orElse(null));
    }

    @NonNull
    private static Stream<Arguments> source() {
        return Stream.generate(RANDOM_DATA::localDateTime).limit(MAX_TEST).map(Arguments::arguments);
    }
}
