/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.dto.kafka;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;

import java.util.stream.Stream;

import static dk.sundhedsdatastyrelsen.cbs.dto.kafka.JobType.FETCH_ALL;
import static dk.sundhedsdatastyrelsen.cbs.dto.kafka.JobType.FETCH_TEST_STATUS;
import static dk.sundhedsdatastyrelsen.cbs.dto.kafka.JobType.FETCH_VACCINATION_STATUS;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.params.provider.Arguments.arguments;

class JobTypeTest {

    @ParameterizedTest
    @DisplayName("bitmask constructor")
    @MethodSource("ofSource")
    void of(@Nullable final JobType expected, @NonNull final JobType... jobTypes) {
        assertEquals(expected, JobType.of(jobTypes));
    }

    @NonNull
    private static Stream<Arguments> ofSource() {
        return Stream.of(
            arguments(null, new JobType[0]),
            arguments(null, new JobType[]{null, null}),
            arguments(FETCH_VACCINATION_STATUS, new JobType[]{FETCH_VACCINATION_STATUS}),
            arguments(FETCH_VACCINATION_STATUS, new JobType[]{FETCH_VACCINATION_STATUS, FETCH_VACCINATION_STATUS}),
            arguments(FETCH_VACCINATION_STATUS, new JobType[]{null, FETCH_VACCINATION_STATUS}),
            arguments(FETCH_TEST_STATUS, new JobType[]{FETCH_TEST_STATUS}),
            arguments(FETCH_TEST_STATUS, new JobType[]{null, FETCH_TEST_STATUS}),
            arguments(FETCH_ALL, new JobType[]{FETCH_ALL}),
            arguments(FETCH_ALL, new JobType[]{null, FETCH_ALL}),
            arguments(FETCH_ALL, new JobType[]{FETCH_TEST_STATUS, FETCH_VACCINATION_STATUS}),
            arguments(FETCH_ALL, new JobType[]{FETCH_TEST_STATUS, FETCH_ALL}),
            arguments(FETCH_ALL, new JobType[]{FETCH_ALL, FETCH_VACCINATION_STATUS}));
    }
}
