/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service;

import dk.sundhedsdatastyrelsen.cbs.dto.kafka.JobType;
import dk.sundhedsdatastyrelsen.cbs.kafka.CbsKafkaProducer;
import dk.sundhedsdatastyrelsen.cbs.monitoring.IsAliveServlet;
import org.apache.log4j.MDC;

import java.util.UUID;

import static dk.sundhedsdatastyrelsen.cbs.CbsLogging.JOB_ID_PROP;

/**
 * Entry point for interacting with the service inside a Spring ApplicationContext
 */
@org.springframework.stereotype.Service
public class Service {
    private final CbsKafkaProducer cbsKafkaProducer;

    public Service(CbsKafkaProducer cbsKafkaProducer) {
        this.cbsKafkaProducer = cbsKafkaProducer;
        IsAliveServlet.setServiceName(ServiceConstants.COMPONENT_NAME);
    }

    /**
     * Creates a DataFetcher job, async (and adds jobId to the logging MDC)
     *
     * @return jobId to be returned in IDWS response, which is the id DataFetcher will use when notifying the job has finished to app-backend via NAS
     */
    public String createDataFetcherJob(String cpr, String idwsMessageId, JobType jobType) {
        String jobId = UUID.randomUUID().toString();
        MDC.put(JOB_ID_PROP, jobId);
        cbsKafkaProducer.sendDataFetcherJob(cpr, jobType, jobId, idwsMessageId);
        return jobId;
    }

}
