/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service.webservice;

import dk.sundhedsdatastyrelsen.cbs.dto.lab.LabResult;
import se.digg.dgc.payload.v1.TestEntry;

import java.time.ZoneOffset;
import java.util.UUID;

import static dk.sundhedsdatastyrelsen.cbs.service.webservice.UUIDUtils.generateType5UUID;

public class LabResultMapper {

    public static final String SNOMED_CT_DETECTED_POSITIVE = "260373001";
    public static final String SNOMED_CT_DETECTED_NEGATIVE = "260415000";

    private static final String NAMESPACE_URL = "6ba7b811-9dad-11d1-80b4-00c04fd430c8";
    private static final UUID namespace = UUIDUtils.generateType5UUID(NAMESPACE_URL, "ssi.dk");

    public static final String LOINC_ANTIGEN = "LP217198-3";
    public static final String LOINC_PCR = "LP6464-4";

    private LabResultMapper() { }

    public static TestEntry toTestEntry(LabResult labResult) {
        TestEntry tst = new TestEntry().withCo("DK")
            .withTg(VaccineMapper.SNOMED_CT_COVID_CONCEPT_ID)
            .withSc(labResult.samplingDateTime().toInstant(ZoneOffset.UTC))
            .withDr(labResult.resultDateTime().toInstant(ZoneOffset.UTC))
            .withTc(labResult.center());

        if (labResult.positive())
            tst.withTr(SNOMED_CT_DETECTED_POSITIVE);
        else
            tst.withTr(SNOMED_CT_DETECTED_NEGATIVE);

        if ("SARSG".equalsIgnoreCase(labResult.type()))
            tst.withTt(LOINC_ANTIGEN);
        else if ("SARS2".equalsIgnoreCase(labResult.type()))
            tst.withTt(LOINC_PCR);
        else
            throw new LabResultMapperException(String.format("Unknown LabResult.type: '%s' - labresult cannot be mapped", labResult.type()));

        tst.setNm(labResult.name());
        tst.setIs(VaccineMapper.ISSUER);
        tst.setCi("urn:uuid:" + generateType5UUID(namespace.toString(), String.valueOf(labResult.id())));
        return tst;
    }
}
