/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.service.webservice;

import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.Vaccination;
import se.digg.dgc.payload.v1.VaccinationEntry;

import java.util.UUID;

public class VaccineMapper {

    public static final String SNOMED_CT_COVID_CONCEPT_ID = "840539006";
    public static final String ISSUER = "Danish Health Data Authority";

    public VaccineMapper(VaccineMappingConfiguration vaccineMappingConfiguration) {
        this.vaccineMappingConfiguration = vaccineMappingConfiguration;
    }

    private final VaccineMappingConfiguration vaccineMappingConfiguration;

    @SuppressWarnings({"squid:S2259"})
    public VaccinationEntry toVac(Vaccination vaccination) {

        if (vaccination.getLatestEffectuated() == null)
            throw new VaccineMapperException("Last effectuated vaccine did not exist");
        VaccineMappingConfiguration.VaccineInformation values = vaccineMappingConfiguration.getVaccineIdValues(vaccination.getLatestEffectuated().vaccineIdentifier());
        if (values == null)
            throw new VaccineMapperException(String.format("Unknown vaccineIdentifier: '%s' - vaccine cannot be mapped", vaccination.getLatestEffectuated().vaccineIdentifier()));

        VaccinationEntry vac = new VaccinationEntry();
        vac.withDt(vaccination.getLatestEffectuated().effectuated().toLocalDate())
            .withTg(SNOMED_CT_COVID_CONCEPT_ID)
            .withVp(values.vp)
            .withMp(values.mp)
            .withMa(values.ma)
            .withCo("DK")
            .withDn(vaccination.vaccinations().size());

        vac.setIs(ISSUER);
        vac.setCi("urn:uuid:" + UUID.randomUUID());

        if (vaccination.completed() != null)
            return vac.withSd(vaccination.vaccinations().size());
        else
            return vac.withSd(vaccination.vaccinations().size() + 1);
    }
}
